# -*- coding: UTF-8 -*-

from common import logger
from upload.converter.atolPlugins import atolformat

errorObjectsCount = 0

def loadObjs(inserter, listObjs, size):
    """ Попытаться загрузить объекты
    Args:
        inserter (Inserter): Плагин, выполняющий загрузку
        listObjs (list[dict[str, str]]): Список объектов, которые нужно загрузить
        size (int): Порог срабатывания загрузки
    """
    if len(listObjs) >= size:
        inserter.add_package(listObjs)
        listObjs[0:] = []  # объекты загружены - очищаем список

def invalidObjectsCount():
    """ Получить количество битых объектов
    Returns:
        int: Количество битых объектов
    """
    return errorObjectsCount

def readDecodedLine(file, charset):
    """ Прочитать строку из файла и преобразовать кодировку в charset
    Args:
        file (IO): Файл для чтения
        charset (str): Кодировка
    Returns:
        str: Считанная строка
    """
    return file.readline().decode(charset)

def general_method(workDirectory, data, node):
    """ Загрузка объектов из файлов, указанных в data
    Args:
        workDirectory (str): Рабочая папка
        data (list[str]): Список файлов для обработки
        node (Node): Информация об узле для загрузки справочников
    Returns:
        bool, list[str]: (Результат вставки aif-объектов, список ошибок)
    """
    global errorObjectsCount
    errorList = []
    formatter = atolformat.AtolFormatter()
    formatter.useMultiplicityQuantityForMeasure = node.currentConverter.useMultiplicityQuantityForMeasure
    formatter.measureMapper = node.currentConverter.measureMapper
    log = logger.getUploadLogger("Node", node.id)
    plugin = __import__('upload.inserter.%s' % node.inserter.name, fromlist='Inserter')
    cl = getattr(plugin, 'Inserter')
    inserter = cl(node)
    result = inserter.start()
    if result:
        for currentFile in data:
            openFile = open(currentFile, 'rb+')  # 'rb' для корректной работы функции file.tell() в Windows
            line = readDecodedLine(openFile, node.currentConverter.charset)
            if not line.strip().startswith('##@@&&'):
                log.error(u'Неверное начало файла %s: %s. Ожидалось: ##@@&&' % (currentFile, line.strip()))
                return False, errorList
            listOfObjs = []
            line = readDecodedLine(openFile, node.currentConverter.charset)
            currentCMD = "$$$ADDQUANTITY" # По-умолчанию включен режим добавления товара с добавлением начального остатка
            if line.strip() == '#':
                position = openFile.tell()
                line = readDecodedLine(openFile, node.currentConverter.charset)
                while line:  # and line.strip().startswith('$$$'): # команд может быть несколько подряд

                    if line.strip().startswith('$$$'):
                        currentCMD = line.strip()
                        if currentCMD in formatter.emptyCommands.keys():
                            aif_obj = formatter.emptyCommands[currentCMD]
                            listOfObjs.extend(aif_obj if isinstance(aif_obj, list) else [aif_obj])
                            loadObjs(inserter, listOfObjs, node.pack_size)
                            currentCMD = "$$$ADDQUANTITY"
                        elif currentCMD not in formatter.commands.keys():
                            line = readDecodedLine(openFile, node.currentConverter.charset)
                            while line and not line.strip().startswith('$$$'):
                                line = readDecodedLine(openFile, node.currentConverter.charset)
                            continue
                    elif not line.strip().startswith('//') and line.strip():  # пропускаем коментарии и пустые строки
                        cmd = formatter.commands[currentCMD]
                        aif_obj = None
                        try:
                            aif_obj = cmd(line)
                        except Exception as e:
                            errorStr = u'Ошибка в строке %s. %s' % (line, e)
                            log.error(errorStr)
                            errorList.append(errorStr)
                        if aif_obj:
                            listOfObjs.extend(aif_obj if isinstance(aif_obj, list) else [aif_obj])
                            loadObjs(inserter, listOfObjs, node.pack_size)
                    line = readDecodedLine(openFile, node.currentConverter.charset)
                if not line:
                    tmpPos = openFile.tell()
                    openFile.seek(position - 2)
                    openFile.write('@'.encode('utf8'))
                    openFile.seek(tmpPos)
            else:
                log.error(u'Неверная вторая строка: %s. Ожидалось "#"' % line)
                openFile.close()
                return False, errorList
            loadObjs(inserter, listOfObjs, 1)
            openFile.close()
        result = inserter.stop()
        errorObjectsCount = inserter.invalidObjectsCount()
    return result, errorList
