# -*- coding: utf-8 -*-

'''
Created on 12.04.2012

@author: kov
'''

import os
import csv
import sys
from artix.common import py2to3utils
from upload.converter.smagPlugins.crc16 import *
from upload.converter.smagPlugins.catalogfactory import CatalogFactory
from common import aif_objects as aif
from common import logger
from upload.inserter import InserterError
if py2to3utils.PY3:
    from dbfread import DBF
else:
    from dbfpy import dbf
    from types import StringType

log = None
packSize = 99
charset = 'cp1251'
errorObjectsCount = 0
setMinPrice = True


def invalidObjectsCount():
    return errorObjectsCount


def run(workDirectory, data, node):
    global errorObjectsCount
    global log, packSize, charset
    global setMinPrice
    errorList = []
    log = logger.getUploadLogger("Node", node.id)
    packSize = node.pack_size
    charset = node.currentConverter.charset
    cf = CatalogFactory()
    setMinPrice = node.currentConverter.setMinPriceIfDiscountRestrictMissing
    log.info(u'Начало работы конвертора')

    action = os.path.splitext(node.currentFlag.lower())[1][1:]
    if action not in ['upd', 'cng']:
        raise AttributeError(u"В обмене формата УКМ флаг загрузки должен иметь расширение 'cng' или 'upd'")
    clear = action == 'cng'

    tables = readFiles(data, node.currentConverter.format)

    result = False
    try:
        plugin = __import__('upload.inserter.%s' % node.inserter.name, fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        inserter = cl(node)
        if inserter.start():
            groupsWithParents = parseGroups(tables, clear, inserter)
            cf.setParams(node.currentConverter, groupsWithParents)
            parseTaxes(tables, clear, inserter)
            parseTmc(tables, clear, inserter, groupsWithParents, cf)
            parseBarcodes(tables, clear, inserter)
            parseAdditionallyPrices(tables, clear, inserter)
            parseDept(tables, clear, inserter)
            parseDiscCard(tables, clear, inserter)
            parsePersonal(tables, clear, inserter, node.currentConverter)

            result = inserter.stop()
            errorObjectsCount = inserter.invalidObjectsCount()
    except InserterError as e:
        raise
    except Exception as e:
        log.exception(u'Ошибка: %s' % e)
        errorList.append(u'Ошибка: %s' % e)

    log.info(u'Окончание работы конвертера')
    return result, errorList


def readFiles(data, formatData):
    log.info(u'Начинаем считывать файлы с данными')

    listTables = ['classif', 'claslim', 'longbar', 'bar', 'clients', 'dclislst',
                          'disccard', 'disccli', 'personal', 'plucash', 'plulim', 'plutax', 'tax']
    tables = {}

    for dictFile in data:
        fileExtension = os.path.split(dictFile)[1]
        tableName = fileExtension.split('.')[0].lower()

        if tableName not in listTables:
            log.warning(u'Неизвестный файл: %s' % tableName)
            continue

        log.info(u'Загружаем файл %s' % tableName)

        if formatData == 'dat':
            if py2to3utils.PY2:
                tables[tableName] = csv.reader(open(dictFile, 'rb'), delimiter=',', quotechar='"')
            else:
                global charset
                tables[tableName] = csv.reader(open(dictFile, 'r', encoding=charset), delimiter=',', quotechar='"')
        else:
            if py2to3utils.PY2:
                tables[tableName] = dbf.Dbf(dictFile)
            else:
                tables[tableName] = DBF(dictFile, encoding=charset, recfactory=lambda row: [value[1] for value in row])
    return tables


def parseGroups(tables, clear, inserter):
    log.info(u'Ищем группы товаров')

    groupsWithParents = {}
    if 'classif' not in tables:
        return groupsWithParents

    listObjects = []
    if clear:
        listObjects.append({"command": "clearInventGroup"})

    countObject = 0
    for group in tables.get("classif", []):
        group = encodeStringList(group)
        countObject += 1

        groupName = group[5]
        groupAsArray = [str(a).rjust(4, '0') for a in group[0:5]]
        groupCode = '.'.join(groupAsArray)

        parents = []
        parent = ''
        while parent != '0000.0000.0000.0000.0000':
            for i in range(4, -1, -1):
                if groupAsArray[i] != '0000':
                    groupAsArray[i] = '0000'
                    break
            parent = '.'.join(groupAsArray)
            parents.append(parent)

        parentGroupCode = parents[0]

        if groupCode == parentGroupCode:
            parentGroupCode = None

        groupsWithParents[groupCode] = parents
        listObjects.append(aif.InventGroup().addCmd(groupcode=groupCode, parentgroupcode=parentGroupCode, groupname=groupName))
        sendPackage(listObjects, inserter)

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)

    return groupsWithParents


def parseTaxes(tables, clear, inserter):
    log.info(u'Ищем налоги')
    if 'tax' not in tables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearLVat"})

    countObject = 0

    plutaxDict = dict((encodeStringList(plutax)[1], encodeStringList(plutax)[2]) for plutax in tables.get('plutax', []))
    for tax in tables.get('tax', []):
        countObject += 1
        tax = encodeStringList(tax)
        taxrate = None
        if tax[0] in plutaxDict:
            taxrate = plutaxDict[tax[0]]
        listObjects.append(aif.LVat().addCmd(code=tax[0], name=tax[2], rate=taxrate))
        sendPackage(listObjects, inserter)
    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s' % countObject)


def parseTmc(tables, clear, inserter, groupsWithParents, cf):
    log.info(u'Ищем товары')
    if 'plucash' not in tables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearInventory"})
        listObjects.append({"command": "clearUnit"})
        listObjects.append({"command": "clearPrice"})

    options = dict()
    options['priceoptions'] = {}
    options['priceoptions']['requiredeferredprice'] = 1
    measure = {}

    pluLim = parsePluLim(tables)
    clasLim = parseClasLim(tables)

    countObject = 0
    plutaxDict = dict((encodeStringList(plutax)[0], encodeStringList(plutax)[1]) for plutax in tables.get('plutax', []))
    for inv in tables.get('plucash', []):
        inv = encodeStringList(inv)
        countObject += 1

        inventCode = inv[0]
        name = inv[1]
        measureName = inv[2]
        fractional = not inv[3] in ['0', '1']
        groupCode = '.'.join([str(a if a else '').rjust(4, '0') for a in inv[11:16]])
        price = inv[16]

        if measureName not in measure:
            measure[measureName] = {'code': crc16(measureName if py2to3utils.PY2 else py2to3utils.ensureBinary(measureName)), 'fractional': fractional}

        taxcode = 0
        if inv[0] in plutaxDict:
            taxcode = plutaxDict[inv[0]]

        deptCode = None if inv[18] in ['', '0'] else inv[18]

        opmode = int(inv[6]) if inv[6] else 0
        obj = aif.Invent().addCmd(inventcode=inventCode, barcode=None, name=name, price=0, deptcode=deptCode, cquant=1,
                                  measurecode=measure[measureName]['code'], inventgroup=groupCode, options=options,
                                  minprice=0, lvatcode=[taxcode], extendedoptions='', opmode=opmode)
        listObjects.append(obj)

        minPrice = getMinPrice(pluLim, clasLim, inventCode, price, groupCode, groupsWithParents)
        listObjects.append(aif.Price().addCmd(barcode=inventCode, price=price, documentid='',
                effectivedate='1970-01-02 00:00:00', doctype=1, minprice=minPrice))

        cf.addInventCatalog(groupCode, inventCode)

        sendPackage(listObjects, inserter)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)

    log.info(u'Ищем товары для каталога')

    listObjects.extend(cf.getAifObjects())

    if listObjects:
        sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % len(cf.getAifObjects()))

    log.info(u'Ищем единицы измерения')
    countObject = 0
    for me in measure:
        countObject += 1
        unit = aif.Unit().addCmd(unitcode=measure[me]['code'], name=me, fractional=measure[me]['fractional'])
        listObjects.append(unit)

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)


def parseBarcodes(tables, clear, inserter):
    log.info(u'Ищем дополнительные штрихкоды товаров')

    tableBarcode = 'bar' if 'bar' in tables else 'longbar'
    if tableBarcode not in tables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearBarcode"})

    countObject = 0
    for bar in tables.get(tableBarcode, []):
        bar = encodeStringList(bar)
        countObject += 1

        barcode = bar[0]
        code = bar[1]
        cquant = 1 if bar[3] == 0 else bar[3]

        listObjects.append(aif.Barcode().addCmd(code=code, barcode=barcode, price=0, name=None, cquant=cquant, measure=None))
        sendPackage(listObjects, inserter)

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)


def parseAdditionallyPrices(tables, clear, inserter):
    log.info(u'Ищем дополнительные цены')

    if 'disccard' not in tables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearAdditionalPrice"})

    countObject = 0
    for dis in tables.get("disccard", []):
        countObject += 1
        dis = encodeStringList(dis)

        barcode = dis[0]
        pricecode = dis[1]
        price = dis[3]

        listObjects.append(aif.AdditionalPrice().addCmd(barcode=barcode, pricecode=pricecode, price=price))
        sendPackage(listObjects, inserter)

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)


def parseDept(tables, clear, inserter):
    log.info(u'Ищем отделы')

    if 'clients' not in tables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearDept"})

    countObject = 0
    for cl in tables.get('clients', []):
        cl = encodeStringList(cl)

        code = cl[0]
        name = cl[2]
        isDept = cl[12] == '2'

        if isDept:
            listObjects.append(aif.Dept().addCmd(code=code, name=name))
            countObject += 1

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)


def parseDiscCard(listTables, clear, inserter):
    log.info(u'Ищем дисконтные карты')

    if 'disccli' not in listTables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearCard"})
        listObjects.append({"command": "clearClient"})

    stopCards = []

    for blockCard in listTables.get('dclislst', []):
        stopCards.append(blockCard[0])

    countObject = 0
    for card in listTables.get('disccli', []):
        card = encodeStringList(card)

        cardNum = card[0]
        clientName = card[1]
        percent = card[2]

        if not cardNum or cardNum[0] == '!' or cardNum in stopCards:
            continue

        countObject += 1

        cardObj = aif.Card().addCmd(idcard=cardNum, idcardgroup=1, idclient=cardNum, number=cardNum,
                                    blocked=0, discountpercent=percent)
        clientObj = aif.Client().addCmd(idclient=cardNum, name=clientName)
        listObjects.append(cardObj)
        listObjects.append(clientObj)
        sendPackage(listObjects, inserter)

    for cardNum in stopCards:
        countObject += 1
        obj = aif.Card().addCmd(idcard=cardNum, idcardgroup=1, number=cardNum, blocked=1)
        listObjects.append(obj)
        sendPackage(listObjects, inserter)

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)


def parsePersonal(tables, clear, inserter, converter):
    log.info(u'Ищем пользователей')

    if 'personal' not in tables:
        return

    listObjects = []
    if clear:
        listObjects.append({"command": "clearMCashUser"})

    countObject = 0
    for person in tables.get("personal", []):
        person = encodeStringList(person)
        countObject += 1

        code = str(person[0])
        name = person[1]
        login = person[0]
        password = person[2]

        roleUsers = [{"rolecode": converter.defaultRoleCode, "rule": "1"}]

        if (converter.relationRolesBy == 'name' and converter.substringInAdminName in name) \
                or (converter.relationRolesBy == 'code' and code in converter.usersWithAdditionallyRole):
            roleUsers.append({"rolecode": converter.additionallyRoleCode, "rule": "1"})

        obj = aif.MCashUser().addCmd(code=code, login=login, name=name, password=password,
                                     locked=0, roleusers=roleUsers)
        listObjects.append(obj)

    sendPackage(listObjects, inserter, True)
    log.info(u'Загружено объектов в очередь: %s.' % countObject)


def parsePluLim(tables):
    pluLim = {}
    for item in tables.get("plulim", []):
        item = encodeStringList(item)

        plInventCode = item[0]
        plPercent = item[1]

        pluLim[plInventCode] = plPercent

    return pluLim


def parseClasLim(tables):
    clasLim = {}
    for item in tables.get("claslim", []):
        item = encodeStringList(item)

        clGroup = '.'.join([str(a).rjust(4, '0') for a in item[0:5]])
        clPercent = item[5]

        clasLim[clGroup] = clPercent

    return clasLim


def getMinPrice(pluLim, clasLim, inventCode, price, groupCode, groupsWithParents):
    minPrice = price if setMinPrice else 0.0
    percent = 0.0
    hasDiscRestrict = False

    if inventCode in pluLim:                    # Ищем процент скидки в pluLim по коду товара
        percent = float(pluLim[inventCode])
        hasDiscRestrict = True
    elif groupCode in clasLim:                  # Ищем процент скидки в clasLim по коду группы товара
        percent = float(clasLim.get(groupCode))
        hasDiscRestrict = True
    else:                                       # Ищем процент скидки в clasLim по родительским кодам групп товара
        if groupCode not in groupsWithParents:
            return minPrice

        for parentGroupCode in groupsWithParents.get(groupCode):
            if parentGroupCode in clasLim:
                percent = float(clasLim.get(parentGroupCode))
                hasDiscRestrict = True
                break

    if hasDiscRestrict and str(price):           # Если ограничения были найдены, высчитываем минимальную цену
        minPrice = float(100.0 - percent) * float(price) / 100.0
        minPrice = float("%10.2f" % minPrice)

    return minPrice


def encodeStringList(stringList):
    global charset
    if py2to3utils.PY2:
        listValues = []

        for elem in stringList:
            if type(elem) == StringType:
                result = unicode(elem, charset)
                elem = result.encode('utf-8')
            listValues.append(elem)

        return listValues
    else:
        return [py2to3utils.ensureStr(elem, encoding='utf-8', safe=True) for elem in stringList]


def sendPackage(listObjects, inserter, anywayAdd=False):
    if len(listObjects) >= packSize or anywayAdd:
        inserter.add_package(listObjects)
        del listObjects[:]
