# -*- coding: UTF-8 -*-

'''
Created on 12.04.2012

@author: kov
'''

from dbfpy import *
from types import *
import sys, os
import csv
import datetime
from common import logger


charset = 'cp1251'
logInfo = "SMAG converter: "
log = None

def run(data, cur, listDefaultTables, format, workDirectory, node):
    global log 
    log = logger.getUploadLogger("Node", node.id)
    
    global charset
    charset = node.currentConverter.charset
#    if node.params.get('charset'):            
#        charset = node.params.get('charset')
    
    currentTables = {}
    for dictFile in data:   
        fileExtension = os.path.split(dictFile)[1]                
        tableName = fileExtension.split('.')[0].lower()
        
        if (tableName in listDefaultTables):
            log.info(logInfo + u"конвертирование %s" % (tableName))                  
            db = None
            if (format == 'dbf'):          
                db = dbf.Dbf(dictFile)
            elif (format == 'dat'):
                db = datInDB(dictFile, tableName, workDirectory)
            
            if tableName != 'personal' and tableName != 'discsum':
                createTable(db, cur, tableName)                
                insertInTable(db, cur, tableName)            
            
            currentTables[tableName] = dictFile
        else:          
            log.error(logInfo + u"неизвестная таблица '%s'" % tableName)
        
    return currentTables 

def datInDB(dictFile, tableName, workDirectory):
    tmpFile = 'tmp.dbf'
    
    if tableName == 'personal':
        tmpFile = 'personal.dbf'
    elif tableName == 'discsum':
        tmpFile = 'discsum.dbf'

    db = dbf.Dbf(workDirectory + "/" + tmpFile, new = True)
    reader = csv.reader(open(dictFile, 'rb'), delimiter=',', quotechar='"')

    db = getDBbyName(tableName, db)
    log.debug(u'Конвертирование dat в dbf (таблица = %s)' % tableName)
    for row in reader:      
        rec = db.newRecord()
        for i in range(len(db.header.fields)):
            field = db.header.fields[i]                
            if row[i] != '':
                if field.typeCode == 'N':                    
                    if field.decimalCount > 0:
                        rec[field.name] = float(row[i])
                    else:
                        rec[field.name] = int(row[i])
                elif  field.typeCode == 'C':
                    rec[field.name] = str(row[i])
                elif  field.typeCode == 'D':
                    str1 = row[i].split('/')
                    rec[field.name] = datetime.date(int(str1[2]), int(str1[1]), int(str1[0]))
        rec.store()
    db.close()
    
    db = dbf.Dbf(workDirectory + "/" + tmpFile)
    log.debug(u'Конвертирование dat в dbf завершено (таблица = %s)' % tableName)
    return db    
def getDBbyName(tableName, db):    
    if tableName == 'classif':
        db.addField(
                    ("GROOP1",  "N", 6),
                    ("GROOP2",  "N", 6),
                    ("GROOP3",  "N", 6),
                    ("GROOP4",  "N", 6),
                    ("GROOP5",  "N", 6),
                    ("NAME",  "C", 80),
                   )
    elif tableName == 'clasdisc':
        db.addField(
                    ("GROOP1", "N", 6),
                    ("GROOP2", "N", 6),
                    ("GROOP3", "N", 6),
                    ("GROOP4", "N", 6),
                    ("GROOP5", "N", 6),
                    ("PRICEINDEX", "N", 6),
                    ("PERCENT", "N", 16, 2),                    
                   )
    elif tableName == 'claslim':
        db.addField(
                    ("GROOP1", "N", 6),
                    ("GROOP2", "N", 6),
                    ("GROOP3", "N", 6),
                    ("GROOP4", "N", 6),
                    ("GROOP5", "N", 6),
                    ("PERCENT", "N", 16, 2),
                   )  
    elif tableName == 'classtax':
        db.addField(
                    ("GROOP1", "N", 6),
                    ("GROOP2", "N", 6),
                    ("GROOP3", "N", 6),
                    ("GROOP4", "N", 6),
                    ("GROOP5", "N", 6),
                    ("TAXINDEX", "N", 6),
                    ("TAX", "N", 16, 2),
                    ("TAXSUMRUB", "N", 16, 2),
                    ("TAXSUMCUR", "N", 16, 2),
                   )
    elif tableName == 'depart':
        db.addField(
                    ("ID", "N", 6),
                    ("NAME", "C", 40),
                   )
    elif tableName == 'longbar' or tableName == 'bar':
        db.addField(
                    ("BARCODE", "C", 15),
                    ("CARDARTICU", "C", 30),
                    ("CARDSIZE", "C", 10),
                    ("QUANTITY", "N", 16, 6),                    
                   )                
    elif tableName == 'cardrate':
        db.addField(
                    ("ID", "N", 6),
                    ("IDENT", "C", 4),
                    ("NAME", "C", 20),
                    ("PRESISION", "N", 16, 6),
                    ("CURS", "N", 16, 2),
                    ("MULCURS", "N", 6),                    
                   )
    elif tableName == 'cliclass':
        db.addField(
                    ("GROOP1",  "N", 6),
                    ("GROOP2",  "N", 6),
                    ("GROOP3",  "N", 6),
                    ("GROOP4",  "N", 6),
                    ("GROOP5",  "N", 6),
                    ("BARCODE", "C", 22),
                    ("PERCENT", "N", 16, 2),                    
                   )
    elif tableName == 'clients':
        db.addField(
                    ("ID", "N", 6),
                    ("IDENT", "C", 20),
                    ("NAME", "C", 40),
                    ("TEL", "C", 40),
                    ("FAX", "C", 40),
                    ("COUNTRY", "C", 20),
                    ("ADDRES", "C", 100),
                    ("MFO", "C", 12),
                    ("ACCOUNT", "C", 15),
                    ("BANK", "C", 100),
                    ("BANKADDRES", "C", 100),
                    ("COMMENT", "C", 254),
                    ("GROOP1", "N", 6),
                    ("GROOP2", "N", 6),
                    ("GROOP3", "N", 6),
                    ("GROOP4", "N", 6),
                    ("GROOP5", "N", 6),
                    ("DELETED", "N", 6),                                        
                   )                
    elif tableName == 'credcard':
        db.addField(
                    ("ID", "N", 6),
                    ("NAME", "C", 80),
                    ("CLIENTINDE",  "N", 6),
                    ("CURINDEX",  "N", 6),
                    ("LIMITSUM",  "N", 16, 2),
                    ("CANRETURN", "N", 6),
                    ("SOFT", "C", 8),
                    ("IDENT", "C", 2),
                   )
    elif tableName == 'credpref':
        db.addField(
                    ("PREFIX", "C", 19),
                    ("CREDCARDIN", "N", 6),
                   )
    elif tableName == 'curs':
        db.addField(
                    ("DATA", "D", 8),
                    ("CURS", "N", 16, 2),
                    ("BANKCURS", "N", 16, 2),
                   )
    elif tableName == 'dclislst':
        db.addField(
                    ("CODEEND", "C", 22),
                    ("CODESTART", "C", 22),
                   )
    elif tableName == 'disccard':
        db.addField(
                    ("CARDARTICU", "C", 30),
                    ("PRICEINDEX", "N", 6),
                    ("QUANTITY", "N", 16, 6),
                    ("PRICERUB", "N", 16, 2),
                    ("PRICECUR", "N", 16, 2),
                    ("DISCOUNT", "N", 16, 2),
                   )
    elif tableName == 'disccli':
        db.addField(
                    ("BARCODE", "C", 22),
                    ("NAME", "C", 40),
                    ("PERCENT", "N", 16, 2),
                    ("CLIENTINDE", "N", 6),
                   )
    elif tableName == 'discsum':
        db.addField(
                    ("PRICEINDEX", "N", 6),
                    ("TIME", "N", 6),
                    ("SUMMA", "N", 16, 2),
                    ("DISCOUNT", "N", 16, 2),
                   )
    elif tableName == 'personal':
        db.addField(
                    ("IDENT", "N", 6),
                    ("NAME", "C", 40),
                    ("PASSW", "C", 15),
                    ("OFFICIALIN", "N", 6),
                   )
    elif tableName == 'plucash':
        db.addField(
                    ("ARTICUL", "C", 30),
                    ("NAME", "C", 80),
                    ("MESURIMENT", "C", 10),
                    ("MESPRESISI", "N", 16, 6),
                    ("ADD1", "C", 20),
                    ("ADD2", "C", 20),
                    ("ADD3", "C", 20),
                    ("ADDNUM1", "N", 16, 6),
                    ("ADDNUM2", "N", 16, 6),
                    ("ADDNUM3", "N", 16, 6),
                    ("SCALE", "C", 10),
                    ("GROOP1", "N", 6),
                    ("GROOP2", "N", 6),
                    ("GROOP3", "N", 6),
                    ("GROOP4", "N", 6),
                    ("GROOP5", "N", 6),
                    ("PRICERUB", "N", 16, 2),
                    ("PRICECUR", "N", 16, 2),
                    ("CLIENTINDE", "N", 6),
                    ("COMMENTARY", "C", 80),
                    ("DELETED", "N", 6),
                    ("MODDATE", "D", 8),
                    ("MODTIME", "N", 6),
                    ("MODPERSONI", "N", 6),
                   )
    elif tableName == 'pludisc' or tableName == 'plugdisc':
        db.addField(
                    ("CARDARTICU", "C", 30),
                    ("CARDSIZE", "C", 10),
                    ("PRICEINDEX", "N", 6),
                    ("PERCENT", "N", 16, 2),
                   )        
    elif tableName == 'plulim':
        db.addField(
                    ("CARDARTICU", "C", 30),
                    ("PERCENT", "N", 16, 2),
                   )
    elif tableName == 'plutax':
        db.addField(
                    ("CARDARTICU", "C", 30),
                    ("TAXINDEX", "N", 6),
                    ("TAX", "N", 16, 2),
                    ("TAXSUMRUB", "N", 16, 2),
                    ("TAXSUMCUR", "N", 16, 2),
                   )        
    elif tableName == 'pricekin':
        db.addField(
                    ("ID", "N", 6),
                    ("NAME", "C", 40),
                   )  
    elif tableName == 'scales':
        db.addField(
                    ("SCALE", "C", 10),
                    ("NAME", "C", 40),                    
                   )
    elif tableName == 'sizes':
        db.addField(
                    ("SCALE", "C", 10),
                    ("CARDSIZE", "C", 10),                    
                   )                                                                      
    elif tableName == 'tax':
        db.addField(
                    ("ID", "N", 6),
                    ("PRIORITY", "N", 6),
                    ("NAME", "C", 40),                    
                   )        
    return db    
    
def createTable(db, cur, tableName):
    log.debug(u'Создание таблицы %s' % tableName)
    request = 'create table %s (' % tableName
    typeField = 'text'    
    for field in sorted(db.header.fields):        
        # "C" (character) "N" (numeric),  "F"(float), "I" (integer), "Y" (currency), "L" (logical), "M" (memofield), "D" (date), "T" (datetime)
        if (field.typeCode == "C"):
            typeField = 'text'
        elif (field.typeCode == "N" or field.typeCode == "Y" or field.typeCode == "T"):
            typeField = 'numeric'
        elif (field.typeCode == "F"):
            typeField = 'real'
        elif (field.typeCode == "I"):
            typeField = 'integer'
        elif (field.typeCode == "L"):
            typeField = 'boolean'
        elif (field.typeCode == "M"):
            typeField = 'none'
        
        request+=field.name + ' ' + typeField + ','
    if (tableName == "plucash"):
        request = request + 'GROOP text,'
        cur.execute("create table plugroop (NAME text, PERCENT numeric, TAX numeric)")

    if (tableName == "claslim"):
        request = request + 'GROOP text,LENGROOP integer,'
    
    if (tableName == "classif"):
        request = request + 'GROOP text,'
        
    if (tableName == "plutax" or tableName == "classtax"):
        request = request + 'IDTAXGROUP text,'
        if (tableName == "classtax"):
            request += 'GROOP text,LENGROOP integer,'

    request = request[:-1] + ')'
    cur.execute(request)
    log.debug(u'Создание таблицы завершено')

def insertInTable(db, cur, tableName):
    log.debug(u'Вставка в таблицу %s' % tableName)
    listGroop = []
    sortedFields = sorted(db.header.fields)
    request = 'insert into %s values (' % tableName
    for i in range(len(sortedFields)):
        request += '?,'
    if (tableName == "plucash" or tableName == "plutax" or tableName == "classif"):
        request += '?,'
    elif (tableName == "claslim"):
            request += '?,?,'
    elif (tableName == "classtax"):
            request += '?,?,?,'
    request = request[0:len(request)-1] + ')'
    
    for rec in db:
        rec = encode(rec)        
        data = []
        for field in sortedFields: 
            data.append(str(rec[field]))
        
        if (tableName == "plucash"):            
            groop = groopToStrFull(rec)
            if not groop in listGroop:
                listGroop.append(groop)            
            data.append(groop)              
        elif (tableName == "disccard"):
            if (data[3] == '0'):        # Если priceindex = 0. То это не доп.цена
                continue                
        elif (tableName == "claslim"):
            groop = groopToStrShort(rec)                            
            data.append(groop)
            data.append(len(groop))                
        elif (tableName == "classif"):
            groop = groopToStrFull(rec)
            data.append(groop)
        elif (tableName == "plutax" or tableName == "classtax"):
            data.append(rec['taxindex'] * 100 + rec['tax'])
            if (tableName == "classtax"):
                gr = groopToStrShort(rec)
                data.append(gr)
                data.append(len(gr))

        cur.execute(request, data)
        
    if len(listGroop):
        for i in listGroop:
            cur.execute("insert into plugroop (name, percent) values('%s', 0)" % i)
    log.debug(u'Вставка в таблицу завершена')


def encode(rec):
                
    for i in range(len(rec.fieldData)):            
        a=rec.fieldData[i]
        if type(a)==StringType:
            a=unicode(a, charset)
            rec.fieldData[i]=a.encode('utf-8')
    return rec

def groopToStrShort(rec):
    groop = ''
    flag = False
    for i in range(5, 0, -1):        
        fieldGroop = str(rec['groop' + str(i)])        
        fieldGroop = "0000"[:-len(fieldGroop)] + fieldGroop
        if (fieldGroop != '0000'):
            flag = True
        if flag:
            groop = fieldGroop + '.' + groop 
    if flag:
        groop = groop[:-1]
    else:        
        groop = "0000"
    return groop

def groopToStrFull(rec):
    groop = ''
    for i in range(1, 6):        
        fieldGroop = str(rec['groop' + str(i)])
        groop = groop + "0000"[:-len(fieldGroop)] + fieldGroop + '.'
    groop = groop[:-1]    
    return groop
