# -*- coding: UTF-8 -*-
import itertools
from itertools import groupby

from common import aif_objects as dicts
from upload.converter.smagPlugins.xml.artix_mapper.ukm_tmc_to_artix_tmc_mapper import UkmToArtixTmcMapper

"""
Вспомогательный класс для получения команд на загрузку aif-объектов
"""


class ConverterUkm(object):

    def __init__(self):
        self.commands = {
            "updateItems": lambda xml_tag: self.tmc_handler(**xml_tag),
            "storePrices": lambda xml_tag: self.price_handler(**xml_tag),
            "storePricesAdd": lambda xml_tag: self.tprice_handler(**xml_tag),
            "taxes": lambda xml_tag: self.tax_handler(**xml_tag),
            "alcocode": lambda xml_tag: self.alcocode_handler(**xml_tag),
            "classif": lambda xml_tag: self.group_handler(**xml_tag),
            "kit": lambda xml_tag: self.kit_handler(**xml_tag)
        }
        self.measureMapper = {}
        self.tnvdMapper = {}

    def get_command(self, line, data, kwargs=None):
        if kwargs is None:
            kwargs = {}
        kwargs['data'] = data
        return self.commands[line](kwargs)

    def tmc_handler(self, data, tnvds):
        tmcs = UkmToArtixTmcMapper.map_tmc(data, tnvds, self.measureMapper, self.tnvdMapper)
        return [dicts.Invent().addCmd(**tmc) for tmc in tmcs]

    @staticmethod
    def alcocode_handler(data):
        alcocodes_gen = UkmToArtixTmcMapper.map_alcocode(data)
        alcocodes, tmc_alcocodes = itertools.tee(alcocodes_gen)
        # на основе полученных алкокодов формируем команды на добавление в таблицы alcocode и tmcalcocode
        add_alcocodes = [dicts.AlcoCode().addCmd(code["alcocode"]) for code in alcocodes]  # type:list
        add_tmc_alcocodes = [dicts.TmcAlcocode().addCmd(**code) for code in tmc_alcocodes]  # type:list
        add_alcocodes.extend(add_tmc_alcocodes)
        return add_alcocodes

    @staticmethod
    def group_handler(data):
        groups = UkmToArtixTmcMapper.map_groups(data)
        return [dicts.InventGroup().addCmd(**group) for group in groups]

    @staticmethod
    def kit_handler(data):
        kits, kit_items = UkmToArtixTmcMapper.map_kit(data)
        # формируем команды на добавление в таблицы tmckit и tmckititem
        kit_add_cmds = [dicts.TmcKit().addCmd(**kit) for kit in kits]  # type: list
        kit_items_add_cmd = [dicts.TmcKitItem().addCmd(**kit_item) for kit_item in kit_items]  # type: list
        kit_add_cmds.extend(kit_items_add_cmd)
        return kit_add_cmds

    @staticmethod
    def price_handler(data):
        prices = UkmToArtixTmcMapper.map_prices(data)
        return [dicts.Price().addCmd(**p) for p in prices]

    @staticmethod
    def tprice_handler(data):
        tprices = UkmToArtixTmcMapper.map_tprices(data)
        return [dicts.AdditionalPrice().addCmd(**p) for p in tprices]

    @staticmethod
    def tax_handler(data):
        taxes = UkmToArtixTmcMapper.map_tax(data)
        # получаем список вида taxgroupid -> [taxes]
        mapped_taxes = groupby(taxes, lambda x: x["code"])
        tax_cmds = [dicts.TaxGroup().addCmd(idtaxgroup=taxgroupid,
                                            taxes=[{'name': tax["name"], 'rate': int(tax["rate"])} for tax in taxes])
                    for taxgroupid, taxes in mapped_taxes]
        return tax_cmds
